import Modifier from 'ember-modifier';
import { begin, end, next } from '@ember/runloop';
import { registerDestructor, isDestroyed, isDestroying } from '@ember/destroyable';
import { tinymce } from '../index.js';

function _defineProperty(obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
}

function _classPrivateFieldGet(receiver, privateMap) {
  var descriptor = _classExtractFieldDescriptor(receiver, privateMap, "get");

  return _classApplyDescriptorGet(receiver, descriptor);
}

function _classPrivateFieldSet(receiver, privateMap, value) {
  var descriptor = _classExtractFieldDescriptor(receiver, privateMap, "set");

  _classApplyDescriptorSet(receiver, descriptor, value);

  return value;
}

function _classExtractFieldDescriptor(receiver, privateMap, action) {
  if (!privateMap.has(receiver)) {
    throw new TypeError("attempted to " + action + " private field on non-instance");
  }

  return privateMap.get(receiver);
}

function _classApplyDescriptorGet(receiver, descriptor) {
  if (descriptor.get) {
    return descriptor.get.call(receiver);
  }

  return descriptor.value;
}

function _classApplyDescriptorSet(receiver, descriptor, value) {
  if (descriptor.set) {
    descriptor.set.call(receiver, value);
  } else {
    if (!descriptor.writable) {
      throw new TypeError("attempted to set read only private field");
    }

    descriptor.value = value;
  }
}

function _checkPrivateRedeclaration(obj, privateCollection) {
  if (privateCollection.has(obj)) {
    throw new TypeError("Cannot initialize the same private elements twice on an object");
  }
}

function _classPrivateFieldInitSpec(obj, privateMap, value) {
  _checkPrivateRedeclaration(obj, privateMap);

  privateMap.set(obj, value);
}

var _customEvents = /*#__PURE__*/new WeakMap();

var _content = /*#__PURE__*/new WeakMap();

var _onEditorContentChange = /*#__PURE__*/new WeakMap();

var _editorEvents = /*#__PURE__*/new WeakMap();

class EditorModifier extends Modifier {
  // Internal list of events that have been bound to the editor
  // Internal string properties used to determined if editor's content has changed
  // Internal boolean property used as a flag to signal end of editor's setContent job
  // Instance of the active editor once initialized

  /**
    List of events given to the editor
     Type of event should be
    ```js
    {
       name: string,
       handler: function
    }
   ```
    @argument customEvents
    @type null|[]
    @default null
   */

  /**
    A given string to be set as editor's content
     @argument content
    @type string
    @default ''
   */

  /**
    A function triggered on every content change.
    This function call might be delayed to avoid multiple content change in the same runloop issue.
     @argument onEditorContentChange
    @type function
    @default undefined
   */

  /**
    A given string representing default bound editor's events
     Each of those events will call an `onEditorContentChange` function if given as parameter
     @argument editorEvents
    @type string
    @default 'change keyup setcontent'
   */
  constructor() {
    super(...arguments);

    _defineProperty(this, "customBoundEvents", []);

    _defineProperty(this, "currentContent", '');

    _defineProperty(this, "currentFormattedContent", '');

    _defineProperty(this, "delayEditorChange", false);

    _defineProperty(this, "editor", null);

    _defineProperty(this, "didSetup", false);

    _classPrivateFieldInitSpec(this, _customEvents, {
      writable: true,
      value: void 0
    });

    _classPrivateFieldInitSpec(this, _content, {
      writable: true,
      value: void 0
    });

    _classPrivateFieldInitSpec(this, _onEditorContentChange, {
      writable: true,
      value: void 0
    });

    _classPrivateFieldInitSpec(this, _editorEvents, {
      writable: true,
      value: void 0
    });

    _defineProperty(this, "cleanup", () => {
      const Editor = this.editor;

      if (Editor) {
        Editor.off('init', this.handleEditorInit.bind(this));
        Editor.off(_classPrivateFieldGet(this, _editorEvents), this.handleEditorChange.bind(this));

        if (this.customBoundEvents.length) {
          this.unbindEditorCustomEvents(Editor);
        }

        Editor.remove();
      }
    });

    registerDestructor(this, this.cleanup);
  }

  modify(element, _, {
    config = {},
    content = '',
    customEvents = [],
    disabled = false,
    editorEvents = 'change keyup setcontent',
    onEditorContentChange
  }) {
    _classPrivateFieldSet(this, _content, content);

    _classPrivateFieldSet(this, _customEvents, customEvents);

    _classPrivateFieldSet(this, _editorEvents, editorEvents);

    _classPrivateFieldSet(this, _onEditorContentChange, onEditorContentChange);

    if (!this.didSetup) {
      // didInstall
      begin();
      const Config = { ...config,
        ...{
          target: element
        },
        setup: editor => {
          this.editor = editor;
          config.setup?.(editor);
          editor.on('init', this.handleEditorInit.bind(this));
          this.setMode(disabled ? 'readonly' : 'design');
        }
      };
      tinymce.init(Config);
      this.didSetup = true;
    } // didReceiveArguments


    const Editor = this.editor;

    if (Editor) {
      this.setMode(disabled ? 'readonly' : 'design');

      if (this.customBoundEvents.length) {
        this.unbindEditorCustomEvents(Editor);
      }

      if (_classPrivateFieldGet(this, _customEvents).length) {
        this.bindEditorCustomEvents(Editor);
      }

      if (_classPrivateFieldGet(this, _content) !== this.currentContent) {
        this.setEditorContent(_classPrivateFieldGet(this, _content));
      }
    }
  }

  setMode(mode) {
    const Editor = this.editor;

    if (Editor.setMode) {
      // Tinymce v5
      Editor.setMode(mode);
    } else {
      // Tinymce v6
      Editor.mode.set(mode);
    }
  }

  handleEditorInit() {
    const Editor = this.editor;

    if (Editor) {
      this.setEditorContent(_classPrivateFieldGet(this, _content));
      Editor.on(_classPrivateFieldGet(this, _editorEvents), this.handleEditorChange.bind(this));

      if (_classPrivateFieldGet(this, _customEvents)) {
        this.bindEditorCustomEvents(Editor);
      }
    }

    end();
  }

  handleEditorChange() {
    const Editor = this.editor;

    if (Editor) {
      if (this.delayEditorChange) {
        /*
         * Editor.setContent is immediately triggered by setEditorContent method
         * Need setEditorContent method to run commpletely before triggering the onEditorContentChange function
         */
        next(() => {
          if (isDestroyed(this) || isDestroying(this)) {
            return;
          }

          this.onHandleEditorChange(Editor);
        });
      } else {
        this.onHandleEditorChange(Editor);
      }
    }
  }

  onHandleEditorChange(Editor) {
    const newContent = Editor.getContent({
      format: 'html'
    });

    if (this.currentFormattedContent !== newContent) {
      this.currentFormattedContent = newContent;
      this.currentContent = newContent;
      _classPrivateFieldGet(this, _onEditorContentChange)?.call(this, newContent ?? '', Editor);
    }
  }
  /*
   * Method to keep track on raw & formatted (by editor) html
   */


  setEditorContent(content) {
    const Editor = this.editor;

    if (Editor) {
      this.delayEditorChange = true;
      Editor.setContent(content);
      this.delayEditorChange = false;
      let formattedContent = Editor.getContent({
        format: 'html'
      });
      this.currentFormattedContent = formattedContent;
      this.currentContent = content;
    }
  }

  bindEditorCustomEvents(Editor) {
    _classPrivateFieldGet(this, _customEvents).forEach(event => {
      this.customBoundEvents.push(event);
      Editor.on(event.name, event.handler);
    });
  }

  unbindEditorCustomEvents(Editor) {
    this.customBoundEvents.forEach(event => {
      Editor.off(event.name, event.handler);
    });
  }

}

export { EditorModifier as default };
//# sourceMappingURL=editor.js.map
